//////////////////////////////////////////////////////////////////////
// geometry.cpp: implementation for the Geometry class.
// -Kyoung Shin Park (2014 Spring)
//////////////////////////////////////////////////////////////////////
#include "geometryPositionNormalTexture.h"


GeometryPositionNormalTexture::GeometryPositionNormalTexture()
{
	//color[0] = randomVal(0.0f, 1.0f); color[1] = randomVal(0.0f, 1.0f); color[2] = randomVal(0.0f, 1.0f);
	numVertices = 0;
	isLoaded = false;
}

Triangle::Triangle(glm::vec3 p_, glm::vec3 v1_, glm::vec3 v2_, glm::vec3 v3_) : GeometryPositionNormalTexture()
{
	set(p_, v1_, v2_, v3_);
}

void Triangle::set(glm::vec3 p_, glm::vec3 v1_, glm::vec3 v2_, glm::vec3 v3_)
{
	p = p_;
	v1 = v1_;
	v2 = v2_;
	v3 = v3_;
	init();
}

void Triangle::init()
{
	glm::vec3 t1 = p + v1;
	glm::vec3 t2 = p + v2;
	glm::vec3 t3 = p + v3;
	glm::vec3 n = glm::cross((t2 - t1), (t3 - t1));


	// LR
	vbo.addData(&v1[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&n[0], sizeof(glm::vec3)); // vertex normal

	// UM
	vbo.addData(&v2[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.5f, 1.0f), sizeof(glm::vec2));
	vbo.addData(&n[0], sizeof(glm::vec3));

	// LL
	vbo.addData(&v3[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2));
	vbo.addData(&n[0], sizeof(glm::vec3));

	numVertices = 3;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);
	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	//int iDataStride = 2 * sizeof(glm::vec3); // vertex & normal only
	int iDataStride = 2 * sizeof(glm::vec3) + sizeof(glm::vec2); // vertex & normal & texCoord
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec2);
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Triangle::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_LOOP, 0, numVertices);
	else
		glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Circle::Circle(glm::vec3 p_, float radius_, int slices_) : GeometryPositionNormalTexture()
{
	set(p_, radius_, slices_);
}

void Circle::set(glm::vec3 p_, float radius_, int slices_)
{
	p = p_; // center position
	radius = radius_;
	slices = slices_;
	init();
}

void Circle::init()
{
	glm::vec3 vertex(0.0f, 0.0f, 0.0f);
	glm::vec3 n(0.0f, 0.0f, 1.0f);
	glm::vec2 texCoord(0.0f, 0.0f);

	float theta = (float) (2*M_PI/slices);
	for (int i=0; i<slices; i++) 
	{
		vertex[0] = p[0] + radius * cos(theta * i);
		vertex[1] = p[1] + radius * sin(theta * i);
		vertex[2] = p[2];
		texCoord[0] = cos(theta * i) / 2.0f + 0.5f;
		texCoord[1] = sin(theta * i) / 2.0f + 0.5f;
		vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position
		vbo.addData(&texCoord[0], sizeof(glm::vec2)); // vertex texture coordnate
		vbo.addData(&n[0], sizeof(glm::vec3)); // vertex normal
	}
	numVertices = slices;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	//int iDataStride = 2 * sizeof(glm::vec3); // vertex & normal only
	int iDataStride = 2 * sizeof(glm::vec3) + sizeof(glm::vec2); // vertex & normal & texCoord
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec2);
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Circle::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_LOOP, 0, numVertices);	// draw wireframe circle  
	else
		glDrawArrays(GL_TRIANGLE_FAN, 0, numVertices);	// draw solid circle  
}

Quad::Quad(glm::vec3 p_, glm::vec3 n_, glm::vec3 u_, float width_, float height_)
{
	set(p_, n_, u_, width_, height_);
}

void Quad::set(glm::vec3 p_, glm::vec3 n_, glm::vec3 u_, float width_, float height_)
{
	p = p_; // center position
	n = n_; // normal vector
	u = u_; // up vector
	width = width_;
	height = height_;
	init();
}

void Quad::init()
{
	glm::vec3 left = glm::cross(n, u);
	glm::vec3 uppercenter = (u * height/2.0f) + p;
	glm::vec3 tl = uppercenter + (left * width/2.0f);
	glm::vec3 tr = uppercenter - (left * width/2.0f);
	glm::vec3 bl = tl - (u * height);
	glm::vec3 br = tr - (u * height);

	// face 1
	vbo.addData(&bl[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&n[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&br[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2));
	vbo.addData(&n[0], sizeof(glm::vec3));

	vbo.addData(&tr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2));
	vbo.addData(&n[0], sizeof(glm::vec3));

	// face 2
	vbo.addData(&bl[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2));
	vbo.addData(&n[0], sizeof(glm::vec3));

	vbo.addData(&tr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2));
	vbo.addData(&n[0], sizeof(glm::vec3));

	vbo.addData(&tl[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2));
	vbo.addData(&n[0], sizeof(glm::vec3));

	numVertices = 6;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	//int iDataStride = 2 * sizeof(glm::vec3); // vertex & normal only
	int iDataStride = 2 * sizeof(glm::vec3) + sizeof(glm::vec2); // vertex & normal & texCoord
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);

	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec2);
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Quad::draw()
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Sphere::Sphere(float radius_, int slices_, int stacks_) : GeometryPositionNormalTexture()
{
	radius = radius_;
	slices = slices_;
	stacks = stacks_;
	init();
}

void Sphere::init()
{
	glm::vec3 vertex(0.0f, 0.0f, 0.0f);
	float lonstep = (float) (M_PI/stacks);
	float latstep = (float) (M_PI/slices);
	float lon, lat;
	glm::vec2 texCoord(0.0f, 0.0f);
	float dstack = 1.0f/stacks;
	float dslice = 1.0f/slices;
	float texX, texY;

	for (lon = 0.0f, texX = 0.0f; lon <= 2*M_PI; lon += (lonstep), texX += (dstack))
	{
		for (lat = 0.0, texY = 0.0f; lat <= M_PI + latstep; lat += (latstep), texY += (dslice))
		{
			vertex[0] = radius * cosf(lon)*sinf(lat);
			vertex[1] = radius * sinf(lon)*sinf(lat);
			vertex[2] = radius * cosf(lat);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position
			texCoord[0] = texX;
			texCoord[1] = texY;
			vbo.addData(&texCoord, sizeof(glm::vec2)); // vertex texture coordnate
			vertex = glm::normalize(vertex);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal
			numVertices++;

			vertex[0] = radius * cosf(lon + lonstep)*sinf(lat);
			vertex[1] = radius * sinf(lon + lonstep)*sinf(lat);
			vertex[2] = radius * cosf(lat);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position
			texCoord[0] = texX + dstack;
			texCoord[1] = texY;
			vbo.addData(&texCoord, sizeof(glm::vec2)); // vertex texture coordnate
			vertex = glm::normalize(vertex);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal
			numVertices++;
		}
	}

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	//int iDataStride = 2 * sizeof(glm::vec3); // vertex & normal
	int iDataStride = 2 * sizeof(glm::vec3) + sizeof(glm::vec2); // vertex & normal & texCoord
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec2);
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Sphere::draw()
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, numVertices);	// draw sphere  
}

Cylinder::Cylinder(float radius_, float height_, int slices_) : GeometryPositionNormalTexture()
{
	radius = radius_;
	height = height_;
	slices = slices_;
	init();
}

void Cylinder::init()
{
	glm::vec3 vertex;
	float xTexCoord = 0.0f;

	float theta = (float) (2*M_PI/slices);
	for (int i = 0; i <= slices; i++) 
	{
		vertex[0] = radius * cosf(theta * i);
		vertex[1] = -height/2.0f;
		vertex[2] = radius * sinf(theta * i);
		vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position
		vbo.addData(&glm::vec2(xTexCoord, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
		vertex[1] = 0.0f;
		vertex = glm::normalize(vertex);
		vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal
		numVertices++;

		vertex[0] = radius * cosf(theta * i);
		vertex[1] = height/2.0f;
		vertex[2] = radius * sinf(theta * i);
		vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position
		vbo.addData(&glm::vec2(xTexCoord, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
		vertex[1] = 0.0f;
		vertex = glm::normalize(vertex);
		vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal
		numVertices++;

		xTexCoord += 1.0f/slices;
	}

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	//int iDataStride = 2 * sizeof(glm::vec3); // vertex & normal
	int iDataStride = 2 * sizeof(glm::vec3) + sizeof(glm::vec2); // vertex & normal & texCoord
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec2);
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	
	isLoaded = true;
}

void Cylinder::draw()
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, numVertices);	// draw cylinder  
}


Cube::Cube(glm::vec3 p_, float size_) : GeometryPositionNormalTexture()
{
	set(p_, size_);
}

void Cube::set(glm::vec3 p_, float size_)
{
	p = p_;
	size = (float) (size_ * 0.5);
	init();
}

void Cube::init()
{
	glm::vec3 frontNormal = glm::vec3(0.0f, 0.0f, 1.0f);
	glm::vec3 backNormal = glm::vec3(0.0f, 0.0f, -1.0f);
	glm::vec3 leftNormal = glm::vec3(-1.0f, 0.0f, 0.0f);
	glm::vec3 rightNormal = glm::vec3(1.0f, 0.0f, 0.0f);
	glm::vec3 bottomNormal = glm::vec3(0.0f, -1.0f, 0.0f);
	glm::vec3 topNormal = glm::vec3(0.0f, 1.0f, 0.0f);
	
	glm::vec3 fll = p + glm::vec3(-size, -size,  size);
	glm::vec3 flr = p + glm::vec3( size, -size,  size);
	glm::vec3 fur = p + glm::vec3( size,  size,  size);
	glm::vec3 ful = p + glm::vec3(-size,  size,  size);
	glm::vec3 bll = p + glm::vec3(-size, -size,  -size);
	glm::vec3 blr = p + glm::vec3( size, -size,  -size);
	glm::vec3 bur = p + glm::vec3( size,  size,  -size);
	glm::vec3 bul = p + glm::vec3(-size,  size,  -size);

	// Front face
	vbo.addData(&fll[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&fur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&fll[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&fur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal

	// Back face
	vbo.addData(&blr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bul[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&blr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bul[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	// Left face
	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&fll[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal
	
	vbo.addData(&bul[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal
	
	// Right face
	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&blr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&fur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	// Bottom face
	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&blr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&fll[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	// Top face
	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&fur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&bul[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	numVertices = 36;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	//int iDataStride = 2 * sizeof(glm::vec3); // vertex & normal
	int iDataStride = 2 * sizeof(glm::vec3) + sizeof(glm::vec2); // vertex & normal & texCoord
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec2);
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Cube::draw()
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Torus::Torus(float radius_, float tubeRadius_, int rings_, int sides_) : GeometryPositionNormalTexture()
{
	radius = radius_;			// radius of torus
	tubeRadius = tubeRadius_;	// radius of tube
	rings = rings_;				// number of radial divisions for torus
	sides = sides_;				// number of tube subdivisions
	init();
}

void Torus::init()
{
	glm::vec3 vertex(0.0f, 0.0f, 0.0f);
   	float theta = (float) (2*M_PI/rings);	// torus outer radius
	float phi = (float) (2*M_PI/sides);		// tube inner radius
	float u, v;
	glm::vec2 texCoord(0.0f, 0.0f);
	float dring = 1.0f/rings;
	float dside = 1.0f/sides;
	float texX, texY;

	for (v = 0.0f, texY = 0.0f; v < 2*M_PI; v += phi, texY += (dside)) 
	{
		for (u = 0.0f, texX = 0.0f; u < 2*M_PI; u += theta, texX += (dring)) 
		{
			vertex[0] = (radius + tubeRadius * cos(v)) * cos(u);
			vertex[1] = (radius + tubeRadius * cos(v)) * sin(u);
			vertex[2] = tubeRadius * sin(v);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position 0
			texCoord[0] = texX;
			texCoord[1] = texY;
			vbo.addData(&texCoord, sizeof(glm::vec2)); // vertex texture coordnate
			vertex = glm::normalize(vertex);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal

			vertex[0] = (radius + tubeRadius * cos(v)) * cos(u + theta);
			vertex[1] = (radius + tubeRadius * cos(v)) * sin(u + theta);
			vertex[2] = tubeRadius * sin(v);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position 1
			texCoord[0] = texX + dring;
			texCoord[1] = texY;
			vbo.addData(&texCoord, sizeof(glm::vec2)); // vertex texture coordnate
			vertex = glm::normalize(vertex);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal

			vertex[0] = (radius + tubeRadius * cos(v + phi)) * cos(u + theta);
			vertex[1] = (radius + tubeRadius * cos(v + phi)) * sin(u + theta);
			vertex[2] = tubeRadius * sin(v + phi);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position 2
			texCoord[0] = texX + dring;
			texCoord[1] = texY + dside;
			vbo.addData(&texCoord, sizeof(glm::vec2)); // vertex texture coordnate
			vertex = glm::normalize(vertex);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal

			vertex[0] = (radius + tubeRadius * cos(v + phi)) * cos(u + theta);
			vertex[1] = (radius + tubeRadius * cos(v + phi)) * sin(u + theta);
			vertex[2] = tubeRadius * sin(v + phi);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position 2
			texCoord[0] = texX + dring;
			texCoord[1] = texY + dside;
			vbo.addData(&texCoord, sizeof(glm::vec2)); // vertex texture coordnate
			vertex = glm::normalize(vertex);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal

			vertex[0] = (radius + tubeRadius * cos(v + phi)) * cos(u);
			vertex[1] = (radius + tubeRadius * cos(v + phi)) * sin(u);
			vertex[2] = tubeRadius * sin(v + phi);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position 3
			texCoord[0] = texX;
			texCoord[1] = texY + dside;
			vbo.addData(&texCoord, sizeof(glm::vec2)); // vertex texture coordnate
			vertex = glm::normalize(vertex);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal

			vertex[0] = (radius + tubeRadius * cos(v)) * cos(u);
			vertex[1] = (radius + tubeRadius * cos(v)) * sin(u);
			vertex[2] = tubeRadius * sin(v);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex position 0
			texCoord[0] = texX;
			texCoord[1] = texY;
			vbo.addData(&texCoord, sizeof(glm::vec2)); // vertex texture coordnate
			vertex = glm::normalize(vertex);
			vbo.addData(&vertex, sizeof(glm::vec3)); // vertex normal

			numVertices += 6;
		}
	}

	fprintf(stderr, "numVertices=%d\n", numVertices);

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	//int iDataStride = 2 * sizeof(glm::vec3); // vertex & normal
	int iDataStride = 2 * sizeof(glm::vec3) + sizeof(glm::vec2); // vertex & normal & texCoord
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec2);
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Torus::draw()
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Parallelpiped::Parallelpiped(glm::vec3 p_, glm::vec3 u_, glm::vec3 v_, glm::vec3 w_) : GeometryPositionNormalTexture()
{
	p = p_;
	u = u_;
	v = v_;
	w = w_;
	init();
}

void Parallelpiped::init()
{
	glm::vec3 pu = p + u;
	glm::vec3 pv = p + v;
	glm::vec3 pw = p + w;
	glm::vec3 puv = p + u + v;
	glm::vec3 pvw = p + v + w;
	glm::vec3 puw = p + u + w;
	glm::vec3 puvw = p + u + v + w;

	glm::vec3 frontNormal = glm::cross(w, u);
	glm::vec3 backNormal = -frontNormal;
	glm::vec3 rightNormal = glm::cross(w, v);
	glm::vec3 leftNormal = -rightNormal;
	glm::vec3 topNormal = glm::cross(v, u);
	glm::vec3 bottomNormal = -topNormal;

	// Front face
	vbo.addData(&p[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&p[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&frontNormal[0], sizeof(glm::vec3)); // vertex normal

	// Back face
	vbo.addData(&puv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&backNormal[0], sizeof(glm::vec3)); // vertex normal

	// Left face
	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&p[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&leftNormal[0], sizeof(glm::vec3)); // vertex normal

	// Right face
	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&rightNormal[0], sizeof(glm::vec3)); // vertex normal

	// Top face
	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pvw[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&topNormal[0], sizeof(glm::vec3)); // vertex normal

	// Bottom face
	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&puv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 0.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(1.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	vbo.addData(&p[0], sizeof(glm::vec3));
	vbo.addData(&glm::vec2(0.0f, 1.0f), sizeof(glm::vec2)); // vertex texture coordnate
	vbo.addData(&bottomNormal[0], sizeof(glm::vec3)); // vertex normal

	numVertices = 36;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	//int iDataStride = 2 * sizeof(glm::vec3); // vertex & normal
	int iDataStride = 2 * sizeof(glm::vec3) + sizeof(glm::vec2); // vertex & normal & texCoord
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec2);
	glEnableVertexAttribArray(2);
	glVertexAttribPointer(2, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Parallelpiped::draw()
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Sa::Sa() : GeometryPositionNormalTexture()
{
	init();
}

void Sa::init()
{
	p[0] = Parallelpiped(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.7f, 1.0f, 0.0f));
	p[1] = Parallelpiped(glm::vec3(1.4f, 0.0f, 0.0f), glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(-0.7f, 1.0f, 0.0f));
	p[2] = Parallelpiped(glm::vec3(2.0f, 0.0f, 0.0f), glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.0f, 1.0f, 0.0f));
	p[3] = Parallelpiped(glm::vec3(2.3f, 0.35f, 0.0f), glm::vec3(0.5f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.0f, 0.3f, 0.0f));
}

void Sa::draw()
{
	for (int i=0; i<4; i++)
		p[i].draw();
}
