#pragma once
//////////////////////////////////////////////////////////////////////
// geometry.h: interface for the Geometry class.
// -Kyoung Shin Park (2013 Spring)
//////////////////////////////////////////////////////////////////////

#include <gl/glew.h>
#include <gl/freeglut.h>
#include <gl/freeglut_ext.h>

#include <glm/glm.hpp>
#include <glm/gtc/type_ptr.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <iostream>

using namespace std;

#include "vertexBufferObject.h"

#ifndef M_PI
#define M_PI (3.1415926535897932384626433832795)
#define DEGTORAD(degree) ((degree) * (M_PI / 180.0f))
#define RADTODEG(radian) ((radian) * (180.0f / M_PI))
#endif
#ifdef WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#define randomVal(min, max) ((float)(min + (max - min) * drand48()))
#endif

class VertexBufferObject;
class GeometryPositionNormalTexture
{
public:
	GeometryPositionNormalTexture();
	virtual ~GeometryPositionNormalTexture() { std::cout << "~Geometry" << std::endl; };
	virtual void init() = NULL;
    virtual void draw() = NULL;
	virtual bool update(float dt) { return true; };
	//const float* getColor() { return &color[0]; };
	//glm::vec3 Color() { return color; };

protected:
	//glm::vec3 color;
	glm::vec3 p; // center position

	int numVertices;
	GLuint vao;
    VertexBufferObject vbo;
	bool isLoaded;
};

class Triangle : public GeometryPositionNormalTexture
{
public:
    Triangle(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 v1=glm::vec3(-1, -1, 0), glm::vec3 v2=glm::vec3(1, -1, 0), glm::vec3 v3=glm::vec3(0, 1, 0));
	void set(glm::vec3 p_, glm::vec3 v1_, glm::vec3 v2_, glm::vec3 v3_);
	void init();
	void draw(bool wireframe=false);
	
private:
	glm::vec3 v1;
	glm::vec3 v2;
	glm::vec3 v3;
};


class Circle : public GeometryPositionNormalTexture
{
public:
	Circle(glm::vec3 p_=glm::vec3(0, 0, 0), float radius=1.0f, int slices=16);
	void set(glm::vec3 p_, float radius_, int slices_);
	void init();
	void draw(bool wireframe=false);

private:
	float radius;
	int slices;
};

class Quad : public GeometryPositionNormalTexture
{
public:
	Quad(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 n=glm::vec3(0, 0, 1), glm::vec3 u=glm::vec3(0, 1, 0), float width=1.0f, float height=1.0f);
	void set(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 n=glm::vec3(0, 0, 1), glm::vec3 u=glm::vec3(0, 1, 0), float width=1.0f, float height=1.0f);
	void init();
	void draw();

private:
	glm::vec3 p; // position
	glm::vec3 n; // normal vector
	glm::vec3 u; // up vector
	float width;
	float height;
};

class Sphere : public GeometryPositionNormalTexture
{
public:
	Sphere(float radius=1.0f, int slices=16, int stacks=16);
	void init();
	void draw();

private:
	float radius;
	int slices;
	int stacks;
};

class Cylinder : public GeometryPositionNormalTexture
{
public:
	Cylinder(float radius=1.0f, float height=1.0f, int slices=16);
	void init();
	void draw();

private:
	float radius;
	float height;
	int slices;
};

class Cube : public GeometryPositionNormalTexture
{
public:
	Cube(glm::vec3 p_=glm::vec3(0, 0, 0), float size=1.0f);
	void set(glm::vec3 p_, float size_);
	void init();
	void draw();

private:
	float size;
};

class Torus : public GeometryPositionNormalTexture
{
public:
	Torus(float radius=1.0f, float tubeRadius=0.25f, int rings=16, int sides=16);
	void init();
	void draw();

private:
	float radius;
	float tubeRadius;
	int rings;
	int sides;
};

class Parallelpiped : public GeometryPositionNormalTexture
{
public:
	Parallelpiped(glm::vec3 p=glm::vec3(0, 0, 0), glm::vec3 u=glm::vec3(1, 0, 0), glm::vec3 v=glm::vec3(0, 0, 1), glm::vec3 w=glm::vec3(0, 1, 0));
	void init();
	void draw();

private:
	glm::vec3 p;
	glm::vec3 u;
	glm::vec3 v;
	glm::vec3 w;
};

class Sa : public GeometryPositionNormalTexture
{
public:
	Sa();
	void init();
	void draw();

private:
	Parallelpiped p[4];
};

