#ifndef __TEXTURE_H__
#define __TEXTURE_H__

enum TextureFiltering
{
	TEXTURE_FILTER_MAG_NEAREST = 0, // Nearest criterion for magnification
	TEXTURE_FILTER_MAG_BILINEAR, // Bilinear criterion for magnification
	TEXTURE_FILTER_MIN_NEAREST, // Nearest criterion for minification
	TEXTURE_FILTER_MIN_BILINEAR, // Bilinear criterion for minification
	TEXTURE_FILTER_MIN_NEAREST_MIPMAP, // Nearest criterion for minification, but on closest mipmap
	TEXTURE_FILTER_MIN_BILINEAR_MIPMAP, // Bilinear criterion for minification, but on closest mipmap
	TEXTURE_FILTER_MIN_TRILINEAR, // Bilinear criterion for minification on two closest mipmaps, then averaged
};

class Texture
{
public:
	Texture(GLuint wrap=GL_REPEAT, GLuint min=GL_LINEAR, GLuint mag=GL_LINEAR);
    virtual ~Texture(void);
    virtual void bind(void) = NULL;
    virtual void disable(void) = NULL;

    void setMinFilter(GLuint);
    void setMagFilter(GLuint);
    void setWrapS(GLuint);
    void setWrapT(GLuint);
    void setWrapR(GLuint);
    void setWrap(GLuint v);
    GLuint getMinFilter(void) const { return minFilter; }
    GLuint getMagFilter(void) const { return magFilter; }
    GLuint getWrapS(void) const { return wrapS; }
    GLuint getWrapT(void) const { return wrapT; }
	GLuint getWrapR(void) const { return wrapR; }

protected:
    GLuint minFilter, magFilter, wrapS, wrapT, wrapR;
};

#endif
