#include <gl/glew.h>
#include <iostream>
using namespace std;

#include "texture2D.h"
#include "simage.h"

Texture2D::Texture2D(int wrap, int min, int mag) :
                Texture(wrap,min,mag)
{
	uiTexture = 0;
	uiSampler = 0;

	imageData = NULL;
	rescaledImageData = NULL;

	printf("Texture2D\n");
}

Texture2D::Texture2D(const char *filename, int wrap, int min, int mag) :
                Texture(wrap,min,mag)
{
	uiTexture = 0;
	uiSampler = 0;

	imageData = NULL;
	rescaledImageData = NULL;

	if (load(filename))
	{
		printf("Texture2D %s is loaded\n", filename);
	}
}

Texture2D::~Texture2D(void)
{
    if (rescaledImageData)
        simage_free_image(rescaledImageData);
    if (imageData)
        simage_free_image(imageData);

	release();
}

bool Texture2D::load(const char * filename, bool mipmaps)
{
	GLsizei xdim2,ydim2;
	GLenum type;
    unsigned char *imgPtr;

	imageData = simage_read_image(filename, &width, &height, &numComponents);
    if (!imageData)
        return false;
	printf("%s is loaded\n", filename);

	xdim2 = 1;
    while (xdim2 <= width)
        xdim2 *= 2;
    xdim2 /= 2;
    ydim2 = 1;
    while (ydim2 <= height)
        ydim2 *= 2;
    ydim2 /= 2;
    if ((width != xdim2) || (height != ydim2))
        {
        rescaledImageData = simage_resize(imageData, width, height, numComponents,
                                           xdim2, ydim2);
        imgPtr = rescaledImageData;
        }
    else
        imgPtr = imageData;

	if (numComponents == 4) type = GL_RGBA;
	else if (numComponents == 3) type = GL_RGB;
	else if (numComponents == 2) type = GL_LUMINANCE_ALPHA;
	else if (numComponents == 1) type = GL_LUMINANCE;

	// create and initialize a texture object
	if (uiTexture == 0)
		glGenTextures(1, &uiTexture);
	glBindTexture(GL_TEXTURE_2D, uiTexture);
	// define texture
	glTexImage2D(GL_TEXTURE_2D, 0, (GLint) type, xdim2, ydim2, 0, type,
                     GL_UNSIGNED_BYTE, imgPtr);
	if (mipmaps) glGenerateMipmap(GL_TEXTURE_2D);
	// set magnification & minification filter
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, magFilter);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, minFilter);
	// set wraping
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, wrapS);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, wrapT);
    glBindTexture(GL_TEXTURE_2D, 0);

	// create and initialize a sampler object
	if (uiSampler == 0)
		glGenSamplers(1, &uiSampler);		
	glSamplerParameteri(uiSampler , GL_TEXTURE_MAG_FILTER , magFilter);  
	glSamplerParameteri(uiSampler , GL_TEXTURE_MIN_FILTER , minFilter);   
	glSamplerParameteri(uiSampler , GL_TEXTURE_WRAP_S, wrapS);   
	glSamplerParameteri(uiSampler , GL_TEXTURE_WRAP_T, wrapT);   

	return true;
}

void Texture2D::subload(GLuint xoffset, GLuint yoffset)
{
    if (!imageData)
        return;

    glBindTexture(GL_TEXTURE_2D, uiTexture);
    if ((minFilter == GL_NEAREST_MIPMAP_NEAREST) ||
          (minFilter == GL_NEAREST_MIPMAP_LINEAR) ||
          (minFilter == GL_LINEAR_MIPMAP_NEAREST) ||
          (minFilter == GL_LINEAR_MIPMAP_LINEAR))
        printf("ERROR: texture2D:subload(): subloading a mipmapped texture"
               " is not supported\n");
    else
        glTexSubImage2D(GL_TEXTURE_2D, 0, xoffset, yoffset,
                        width, height, GL_RGBA, GL_UNSIGNED_BYTE, imageData);
    glBindTexture(GL_TEXTURE_2D,0);
}

void Texture2D::bind(void)
{
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, uiSampler);
	glBindSampler(0, uiSampler);// set sampler state
}

void Texture2D::disable(void)
{
	glBindTexture(GL_TEXTURE_2D, 0);
    glDisable(GL_TEXTURE_2D);
	glBindSampler(0, 0);
}

void Texture2D::release(void)
{
	glDeleteTextures(1, &uiTexture);
	glDeleteSamplers(1, &uiSampler);
}

void Texture2D::setSamplerParameter(GLenum parameter, GLenum value)
{
	glSamplerParameteri(uiSampler, parameter, value);
}

