#include <cmath>
#include <iostream>
#include <GL/glew.h>
#include <GL/freeglut.h>
#include <GL/freeglut_ext.h>
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <vector>
#include "simage.h"
#include "texture2D.h"
#include "geometryPositionNormalTexture.h"

// Square vertices
std::vector<glm::vec3> squareVertices;
// Square textures
std::vector<glm::vec2> squareTextureCoords;
// Indices into squareVerts[] buffer for 2 triangles
std::vector<int> squareIndices;
// Square VAO
GLuint squareVAO;
// Shader
GLuint programID;

// wireframe/filled
bool g_wiremode = false;

/*
// sampler binding
GLuint textureID = 0;   
GLuint sampler = 0;   
GLuint texture = 0;
*/

GLuint textureID = 0;   
Texture2D * tex = NULL;
Texture2D * tex1[6];

Quad * square = NULL;
std::vector<GeometryPositionNormalTexture*> geoList;
int g_geometrymode = 0;
glm::mat4 yRotation(1.0f);

////////////////////////////////////////////////////////////////////////////////////////////////////

// Create a NULL-terminated string by reading the provided file
static char* readShaderSource(const char* shaderFile)
{
    FILE* fp = fopen(shaderFile, "rb");

    if ( fp == NULL ) { return NULL; }

    fseek(fp, 0L, SEEK_END);
    long size = ftell(fp);

    fseek(fp, 0L, SEEK_SET);
    char* buf = new char[size + 1];
    fread(buf, 1, size, fp);

    buf[size] = '\0';
    fclose(fp);

    return buf;
}

// Create a GLSL program object from vertex and fragment shader files
GLuint InitShader(const char* vShaderFile, const char* fShaderFile)
{
    struct Shader {
	const char*  filename;
	GLenum       type;
	GLchar*      source;
    }  shaders[2] = {
	{ vShaderFile, GL_VERTEX_SHADER, NULL },
	{ fShaderFile, GL_FRAGMENT_SHADER, NULL }
    };

    GLuint program = glCreateProgram();
    
    for ( int i = 0; i < 2; ++i ) {
	Shader& s = shaders[i];
	s.source = readShaderSource( s.filename );
	if ( shaders[i].source == NULL ) {
	    std::cerr << "Failed to read " << s.filename << std::endl;
	    exit( EXIT_FAILURE );
	}

	GLuint shader = glCreateShader( s.type );

	glShaderSource( shader, 1, (const GLchar**) &s.source, NULL );
	glCompileShader( shader );

	GLint  compiled;
	glGetShaderiv( shader, GL_COMPILE_STATUS, &compiled );
	if ( !compiled ) {
	    std::cerr << s.filename << " failed to compile:" << std::endl;
	    GLint  logSize;
	    glGetShaderiv( shader, GL_INFO_LOG_LENGTH, &logSize );
	    char* logMsg = new char[logSize];
	    glGetShaderInfoLog( shader, logSize, NULL, logMsg );
	    std::cerr << logMsg << std::endl;
	    delete [] logMsg;

	    exit( EXIT_FAILURE );
	}

	delete [] s.source;

	glAttachShader( program, shader );
    }

    /* link  and error check */
    glLinkProgram(program);

    GLint  linked;
    glGetProgramiv( program, GL_LINK_STATUS, &linked );
    if ( !linked ) {
	std::cerr << "Shader program failed to link" << std::endl;
	GLint  logSize;
	glGetProgramiv( program, GL_INFO_LOG_LENGTH, &logSize);
	char* logMsg = new char[logSize];
	glGetProgramInfoLog( program, logSize, NULL, logMsg );
	std::cerr << logMsg << std::endl;
	delete [] logMsg;

	exit( EXIT_FAILURE );
    }

    /* use program object */
    glUseProgram(program);

    return program;
}

GLuint SetSquareData()
{
	// Vertice Positions
	squareVertices.push_back(glm::vec3(-0.75f, -0.75f, 0.0f));
	squareVertices.push_back(glm::vec3(0.75f, -0.75f, 0.0f));
	squareVertices.push_back(glm::vec3(0.75f, 0.75f, 0.0f));
	squareVertices.push_back(glm::vec3(-0.75f, 0.75f, 0.0f));
	// Vertices Textures
	squareTextureCoords.push_back(glm::vec2(0.0f, 0.0f));
	squareTextureCoords.push_back(glm::vec2(1.0f, 0.0f));
	squareTextureCoords.push_back(glm::vec2(1.0f, 1.0f));
	squareTextureCoords.push_back(glm::vec2(0.0f, 1.0f));
	// Vertice Indices
	squareIndices.push_back(0);
	squareIndices.push_back(1);
	squareIndices.push_back(2);
	squareIndices.push_back(0);
	squareIndices.push_back(2);
	squareIndices.push_back(3);

	// VAO
	GLuint vao;
	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);

	// VBO
	GLuint vbo[2];
	glGenBuffers(2, &vbo[0]);
	glBindBuffer(GL_ARRAY_BUFFER, vbo[0]);
	glBufferData(GL_ARRAY_BUFFER, 4*sizeof(glm::vec3), &squareVertices[0], GL_STATIC_DRAW);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(0);
	glBindBuffer(GL_ARRAY_BUFFER, vbo[1]);
	glBufferData(GL_ARRAY_BUFFER, 4*sizeof(glm::vec2), &squareTextureCoords[0], GL_STATIC_DRAW);
	glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, 0);
	glEnableVertexAttribArray(1);

	// IBO
	GLuint ibo;
	glGenBuffers(1, &ibo);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, 6*sizeof(int), &squareIndices[0], GL_STATIC_DRAW);

	squareVertices.clear();
	squareTextureCoords.clear();

	return vao;
}

void init( void )
{
    // Load shaders and use the resulting shader program
    programID = InitShader( "TransformVertexShader.vert", "TextureFragmentShader.frag" );
	glUseProgram(programID);

	// Set sampler data
	//tex = new CTexture();
	tex = new Texture2D();
	tex->load("tree01S.png");
	
	tex1[0] = new Texture2D();
	tex1[0]->load("rainbow.png");
	tex1[1] = new Texture2D();
	tex1[1]->load("tree02S.png");
	tex1[2] = new Texture2D();
	tex1[2]->load("earth.jpg");
	tex1[3] = new Texture2D();
	tex1[3]->load("tree35S.png");
	tex1[4] = new Texture2D();
	tex1[4]->load("moon.jpg");
	tex1[5] = new Texture2D();
	tex1[5]->load("main.jpg");

	// set geometry
	square = new Quad(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(0.0f, 1.0f, 0.0f), 3.0f, 3.0f);
	geoList.push_back(new Quad(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(0.0f, 1.0f, 0.0f), 3.0f, 3.0f));
	geoList.push_back(new Cube(glm::vec3(0.0f, 0.0f, 0.0f), 1.5f));
	geoList.push_back(new Sphere(1.5f, 16, 16));
	geoList.push_back(new Cylinder(1.0f, 2.0f, 32));
	geoList.push_back(new Torus(1.0f, 0.1f, 32, 16));
	geoList.push_back(new Parallelpiped(glm::vec3(0.0f, 0.0f, 0.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(1.5f, 2.0f, 0.0f)));

	squareVAO = SetSquareData();

	glEnable(GL_DEPTH_TEST);
	//glDepthFunc(GL_LESS);
	glDepthFunc(GL_LEQUAL);	// accept co-planar fragments
	glClearColor( 0.5, 0.5, 0.5, 1.0 ); // black background
}

void display( void )
{
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );     // clear the window

    glUseProgram(programID);

	// Projection matrix : 45?Field of View, 4:3 ratio, display range : 0.1 unit <-> 100 units
	glm::mat4 Projection = glm::perspective(45.0f, 1.0f, 0.1f, 100.0f);
	// Camera matrix
	glm::mat4 View       = glm::lookAt(
								glm::vec3(4,3,3), // Camera is at (4,3,3), in World Space
								glm::vec3(0,0,0), // and looks at the origin
								glm::vec3(0,1,0)  // Head is up (set to 0,-1,0 to look upside-down)
						   );
	// Model matrix : an identity matrix (model will be at the origin)
	glm::mat4 Model      = glm::translate(glm::mat4(1.0f), glm::vec3(0, 0, -2)); // glm::mat4(1.0f);
	// Our ModelViewProjection : multiplication of our 3 matrices
	glm::mat4 MVP        = Projection * View * Model; // Remember, matrix multiplication is the other way around
	// Get a handle for our "MVP" uniform
	GLuint MatrixID = glGetUniformLocation(programID, "gMVP");
	glUniformMatrix4fv(MatrixID, 1, GL_FALSE, &MVP[0][0]);

	// Set our "gTextureSampler" sampler to user Texture Unit 0
	//textureID  = glGetUniformLocation(programID, "gTextureSampler");
	//glUniform1i(textureID, 0);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	tex->bind();
	square->draw();
	tex->disable();
	glDisable(GL_BLEND);

	// Model matrix : an identity matrix (model will be at the origin)
	Model      = yRotation; // glm::mat4(1.0f);
	// Our ModelViewProjection : multiplication of our 3 matrices
	MVP        = Projection * View * Model; // Remember, matrix multiplication is the other way around
	MatrixID = glGetUniformLocation(programID, "gMVP");
	glUniformMatrix4fv(MatrixID, 1, GL_FALSE, &MVP[0][0]);

	glDepthMask(GL_FALSE); 
	glEnable(GL_CULL_FACE); 
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	tex1[g_geometrymode]->bind();
	geoList[g_geometrymode]->draw();
	tex1[g_geometrymode]->disable();
	glDisable(GL_BLEND);
		/*
	// square
	glBindVertexArray(squareVAO);
	if (g_wiremode) 
		glDrawElements(GL_LINE_LOOP, 6, GL_UNSIGNED_INT, 0);       // draw wireframe
	else
		glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);       // draw filled
	glBindVertexArray(0);
	glDisableVertexAttribArray(0);
	glDisableVertexAttribArray(1);
	*/
	glutSwapBuffers();
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0; 
	static int startTime, elapsedTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;
	prevTime = currentTime;

    // translate & rotate around x/y axis
	static float angle = 0.0f;
	angle += 0.1f;
	glm::mat4 xTranslate, xTranslate2, xTranslate3;
	glm::mat4 xRotation, xRotation2, xRotation3;

	// 1st geometry translate & rotate in x & rotate in y
	xTranslate = glm::translate(glm::mat4(1.0f), glm::vec3(1, 0, 0)); 
	xRotation = glm::rotate(xTranslate, angle, glm::vec3(1, 0, 0));
	yRotation = glm::rotate(xRotation, angle, glm::vec3(0, 1, 0));
	
	elapsedTime = currentTime - startTime;
	glutPostRedisplay();
}

void keyboard( unsigned char key, int x, int y )
{
	switch (key)  
	{    
	case 0x1B:    
	case 'q':    
	case 'Q':      
		exit(0);     
	case ' ':
		g_wiremode = !g_wiremode;
		break;  
	case 'g':
		g_geometrymode = (g_geometrymode+1) % 6 ;
		printf("g_geometrymode=%d\n", g_geometrymode);
		break;
	}
	glutPostRedisplay();
}

int main( int argc, char **argv )
{
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
    glutInitWindowSize( 512, 512 );
    glutCreateWindow(argv[0]);
    
	glewInit();
    init();

    glutDisplayFunc( display );
    glutKeyboardFunc( keyboard );
	glutIdleFunc( update );

    glutMainLoop();
    return 0;
}
